import { juxt, pipe, props, equals, filter, not, compose, ifElse, T, reject, all, apply, prop } from 'ramda'
import isPath from './isPath'
import isOperation from './isOperation'
import isValueForOperation from './isValueForOperation'

/**
 * Tests if a given object is a valid JSON Patch object
 * according to https://tools.ietf.org/html/rfc6902
 *
 * @param {object} jsonPatch - the json patch to be tested
 * @return {boolean} testResult - true|false if the object is a valid JSON Patch
 * @example
 *
 * let patch1 = { op: 'add', path: '/foo', value: 'bar' }
 * let patch2 = { op: 'add', path: 'foo', value: 'bar' }
 * isPatch(patch1) // true
 * isPatch(patch2) // false
 *
 */
const validatePatch = juxt([
  pipe(prop('path'), isPath),
  pipe(prop('op'), isOperation),
  pipe(
    props(['op', 'value', 'from']),
    filter(pipe(equals(undefined), not)),
    apply(isValueForOperation)
  )
])

const isPatch = compose(
  ifElse(all(equals(true)), T, reject(equals(true))),
  validatePatch
)

export default isPatch
